"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.isIterable = exports.curryIterFunction = exports.stripIterable = exports.kComb = void 0;
/**
 * Creates a function which always returns the same value.
 * @param value - The value to be returned
 * @typeParam T - The type of the constant
 * @internal
 */
function kComb(value) {
    return () => value;
}
exports.kComb = kComb;
const underlying = Symbol();
class StrippedIterable {
    constructor(original) {
        Object.defineProperty(this, _a, {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this[underlying] = original;
    }
    [(_a = underlying, Symbol.iterator)]() {
        return this[underlying][Symbol.iterator]();
    }
}
/**
 * Strips an iterable of all other properties so it only contains the iterator
 * symbol
 * @param it - The iterator to be stripped.
 * @typeParam T - The iterable's item type.
 * @returns An iterable object stripped of all other properties.
 * @internal
 */
function stripIterable(it) {
    return new StrippedIterable(it);
}
exports.stripIterable = stripIterable;
/**
 * Curries a iter function for partial application.
 * @param function - The function to curry.
 * @typeParam T - The iterable's item type.
 * @typeParam U - The function's return type.
 * @typeParam Args - The other argument types for `function` as a tuple.
 * @returns A curried `function`.
 * @internal
 */
function curryIterFunction(f) {
    return (...args) => (it) => f(it, ...args);
}
exports.curryIterFunction = curryIterFunction;
/**
 * Check if a value is an iterable.
 * @param x - The value to be checked
 * @returns Whether the value implements `Iterable`
 */
// deno-lint-ignore no-explicit-any
function isIterable(x) {
    return typeof x[Symbol.iterator] === "function";
}
exports.isIterable = isIterable;
